@@ -31,47 +31,62 @@ class ReHeader (object):
                 'Standard input is supported as source only for header templates, not for the original bam format.')
 
         try:
-            ori_header_lines = list(pysamtools.header(orifile, 'bam'))
-        except AssertionError:
+            ori_header = samheader.Header.frombam(orifile)
+        except RuntimeError:
             try:
-                ori_header_lines = list(pysamtools.header(orifile, 'sam'))
-            except AssertionError:
-                raise AssertionError('Need original file in SAM/BAM format.')
+                ori_header = samheader.Header.fromsam(orifile)
+            except RuntimeError:
+                raise RuntimeError('Need original file in SAM/BAM format.')
 
         self.rg_mapping = rg_mapping
         self.sq_mapping = sq_mapping
-        self.original = samheader.Header(ori_header_lines)
-        self.new = samheader.Header(ori_header_lines)
+        self.original = ori_header
+        self.new = self.original.copy()
         
         if any(
             param == '-' for param in (t_default, rg_specs.template,
                                        sq_specs.template, co_specs.template)):
-            stdin_hdr = samheader.Header.fromsam('-')
+            try:
+                stdin_hdr = samheader.Header.fromsam('-')
+            except RuntimeError:
+                raise RuntimeError('Need SAM format from standard input.')
             
         if t_default:
             if t_default == '-':
                 hdr_new = stdin_hdr
             else:
-                hdr_new = t_default if isinstance(t_default, samheader.Header) else samheader.Header.fromsam(t_default)
+                try:
+                    hdr_new = t_default if isinstance(t_default, samheader.Header) else samheader.Header.fromsam(t_default)
+                except RuntimeError:
+                    raise RuntimeError('Need header template in SAM format.')
         else:
             hdr_new = samheader.Header()
         if rg_specs.template:
             if rg_specs.template == '-':
                 hdr_rg = stdin_hdr
             else:
-                hdr_rg = rg_specs.template if isinstance(rg_specs.template, samheader.Header) else samheader.Header.fromsam(rg_specs.template)
+                try:
+                    hdr_rg = rg_specs.template if isinstance(rg_specs.template, samheader.Header) else samheader.Header.fromsam(rg_specs.template)
+                except RuntimeError:
+                    raise RuntimeError('Need RG section template in SAM format.')
             hdr_new.merge_rg(hdr_rg, treat = 'replace')
         if sq_specs.template:
             if sq_specs.template == '-':
                 hdr_sq = stdin_hdr
             else:
-                hdr_sq = sq_specs.template if isinstance(sq_specs.template, samheader.Header) else samheader.Header.fromsam(sq_specs.template)
+                try:
+                    hdr_sq = sq_specs.template if isinstance(sq_specs.template, samheader.Header) else samheader.Header.fromsam(sq_specs.template)
+                except RuntimeError:
+                    raise RuntimeError('Need SQ section template in SAM format.')
             hdr_new.merge_sq(hdr_sq, treat = 'replace')
         if co_specs.template:
             if co_specs.template == '-':
                 hdr_co = stdin_hdr
             else:
-                hdr_co = co_specs.template if isinstance(co_specs.template, samheader.Header) else samheader.Header.fromsam(co_specs.template)
+                try:
+                    hdr_co = co_specs.template if isinstance(co_specs.template, samheader.Header) else samheader.Header.fromsam(co_specs.template)
+                except RuntimeError:
+                    raise RuntimeError('Need comment section template in SAM format.')
             hdr_new.merge_co(hdr_co, treat = 'replace')
         if rg_specs.mode:
             self.new.merge_rg(hdr_new, treat = rg_specs.mode, mapping = rg_specs.id_mapping)
@@ -191,16 +206,19 @@ def fq2sam (filelist1, filelist2, outputname=None, inputcomp=True, outputcomp=Tr
         if isinstance (header, samheader.Header):
             new_header = header
         else:
-            new_header = samheader.Header.fromsam(header)
+            try:
+                new_header = samheader.Header.fromsam(header)
+            except RuntimeError:
+                raise RuntimeError('The header file must have SAM format.')
     else:
         new_header = samheader.Header()
 
     # add yourself to the @PG lines
-    new_header['PG'].append(
-                    {'ID':'MiModD', 'PN':'mimodd convert',
-                     'CL': ' '.join(sys.argv),
-                     'VN': str(mimodd_base.__version__)
-                    })
+    # new_header['PG'].append(
+    #                {'ID':'MiModD', 'PN':'mimodd convert',
+    #                 'CL': ' '.join(sys.argv),
+    #                 'VN': str(mimodd_base.__version__)
+    #                })
     # if no read group was specified, use default
     if not new_header['RG']:
         new_header['RG']=[{'ID':'000'}]
@@ -303,6 +321,9 @@ def convert (input1, input2 = None, iformat = 'bam', oformat = 'sam',
                  'gz':('sam', 'bam'),
                  'sam':('sam', 'bam',),
                  'bam':('sam', 'bam')}
+    if iformat not in supported:
+        raise ValueError('Unknown input format {0}. Acceptable values: {1}'.
+		 format(iformat, ', '.join(sorted(supported))))
     if oformat not in supported[iformat]:
         raise ValueError('Conversion from {0} to {1} is not currently supported.'.format(iformat, oformat))
 
@@ -318,7 +339,7 @@ def convert (input1, input2 = None, iformat = 'bam', oformat = 'sam',
             raise ValueError('header argument is not allowed with SAM/BAM input. Use the reheader tool to change header information.')
         sam2bam(input1[0], outputname, iformat, oformat)
     else:
-        raise ValueError('Input format {0} is not supported.'.format(iformat))
+        raise AssertionError('Oh oh, this looks like a bug')
 
 def reheader (inputfile, rg_specs, sq_specs, co_specs,
               template = None, outputfile = None,  
