@@ -21,8 +21,7 @@ from .decox import parachute, FuncInfo
 from .fasta import FastaReader
 
 tempfile_dir = config.tmpfiles_path
-snap_exe = os.path.normpath(
-    os.path.join(config.__file__, config.bin_path, 'snap'))
+snap_exe = config.snap_exe
 
 # raise an error on SIGTERM throughout this module
 signal.signal(signal.SIGTERM, mimodd_base.catch_sigterm)
@@ -58,7 +57,7 @@ def snap_call (mode, refgenome_or_indexdir, inputfiles, iformat = 'fastq', oform
              snap index; default: config.tmpfiles_path/outfileprefix+'_index'
     idx_seedsize: the -s parameter for snap index
     idx_slack: the -h parameter for snap index
-    
+    idx_ofactor: the -O parameter for snap index
     """
     
     # snap_call_argcheck does basic argument validation and returns a flattened dictionary of all arguments for convenience
@@ -172,8 +171,11 @@ def snap_call (mode, refgenome_or_indexdir, inputfiles, iformat = 'fastq', oform
                 else:
                     ref_index = os.path.join(tempfile_dir, outfileprefix + '_index')
                     idx_remove = True
-                call, results, errors = snap_index(refgenome_or_indexdir, ref_index,
-                                                   args.get('idx_seedsize'), args.get('idx_slack'), verbose = verbose, quiet = quiet)
+                call, results, errors = snap_index(
+                                            refgenome_or_indexdir, ref_index,
+                                            args.get('idx_seedsize'), args.get('idx_slack'),
+                                            args.get('idx_ofactor'), args.get('threads'),
+                                            verbose = verbose, quiet = quiet)
             # get the md5sums of all sequences just indexed
             with open(os.path.join(ref_index, 'md5sums'), 'r') as ifo:
                 md5dict = {seqtitle: md5 for seqtitle, md5 in (line.strip().split('\t') for line in ifo)}
@@ -231,7 +233,12 @@ def snap_call (mode, refgenome_or_indexdir, inputfiles, iformat = 'fastq', oform
         header['SQ'] = snap_header['SQ']          # SNAP gets the sequences right,
         for sq in header['SQ']:                   # but we want to add MD5 tags to them
             sq['M5'] = md5dict[sq['SN']]
-        header['PG'].extend(snap_header['PG'])    # add SNAP's PG header line
+        # modify and add SNAP's PG header line
+        for pg in snap_header['PG']:
+            if pg['ID'] == 'SNAP':
+                pg['PN'] = 'MiModD snap'
+                pg['VN'] = str(mimodd_base.__version__)
+        header['PG'].extend(snap_header['PG'])
             
         rg_id = header['RG'][0]['ID'] # we need the RG ID value to write it to each read's tag list below
 
@@ -345,7 +352,7 @@ def snap_call_argcheck(**args):
     if args['iformat'] in ('sam', 'bam'):
         try:
             header = samheader.Header.fromfile(args['inputfiles'][0], args.get('iformat'))
-        except AssertionError:
+        except RuntimeError:
             pass
         if len(header.get('RG', [])) > 1:
             raise RuntimeError('Multiple read groups declared in input file {0}. Such input is not currently supported.'
@@ -355,7 +362,7 @@ def snap_call_argcheck(**args):
         if not isinstance (custom_header, samheader.Header):
             try:
                 custom_header = samheader.Header.fromsam(custom_header)
-            except:
+            except RuntimeError:
                 raise ValueError('Unable to get header information from custom header argument for file {0}.'
                                  .format(args['inputfiles'][0]))
         if len(custom_header['RG']) != 1:
@@ -424,10 +431,10 @@ def snap_batch (job_arglist):
                 idx_key = (idx_settings[0], idx_settings[1] or '_'.join((unique_output_prefix, os.path.basename(idx_settings[0]).split('.')[0], str(idx_settings[2] or 0), str(idx_settings[3] or 0))),
                            idx_settings[2], idx_settings[3], False if idx_settings[1] else True)
                 if not os.path.exists(idx_key[1]):
-                    call, results, errors = snap_index(*idx_key[:-1], verbose = job_args["verbose"], quiet = job_args["quiet"])
+                    call, results, errors = snap_index(*idx_key[:-1], ofactor = job_args.get('idx_ofactor'), threads = job_args.get('threads'), verbose = job_args["verbose"], quiet = job_args["quiet"])
                 refgenomes[idx_key] -= 1
                 job_args['refgenome_or_indexdir'] = idx_key[1]
-                for param in ('idx_out', 'idx_seedsize', 'idx_slack'):
+                for param in ('idx_out', 'idx_seedsize', 'idx_slack', 'idx_ofactor'):
                     if param in job_args:
                         del job_args[param]
             # keep track of temporary files generated
@@ -491,7 +498,9 @@ def make_snap_argdictlist (commands=None, ifile=None):
     job_arglist = []
     try:
         if not commands:
-            assert ifile, 'Either the commands or the ifile argument MUST be specified.'
+            if not ifile:
+                raise ValueError(
+                    'Either the commands or the ifile argument MUST be specified.')
             commands = (line for line in open(ifile) if line.strip())
         for line_no, command in enumerate(commands, 1):
             try:
@@ -509,7 +518,8 @@ def make_snap_argdictlist (commands=None, ifile=None):
             for hdr in all_headers:
                 for rg in hdr['RG']:
                     if rg['ID'] in rg_sm_mapping:
-                        assert rg_sm_mapping[rg['ID']] == rg['SM'], 'The input files declare the read group {0} more than once with different sample names.'.format(rg['ID'])
+                         if rg_sm_mapping[rg['ID']] != rg['SM']:
+                             raise RuntimeError('The input files declare the read group {0} more than once with different sample names.'.format(rg['ID']))
                     else:
                         rg_sm_mapping[rg['ID']] = rg['SM']
             return job_arglist
@@ -533,37 +543,37 @@ SnapCLParser.add_argument('refgenome_or_indexdir', metavar = 'reference genome o
 SnapCLParser.add_argument("inputfiles", metavar = 'input file(s)', nargs = "+", help = "one or two (in 'paired' mode with 'fastq' input format) input files")
 # optionals (optional arguments)
 # defaulting to argparse.SUPPRESS prevents function definition defaults from being overwritten
-SnapCLParser.add_argument("-o", "--outputfile", metavar = 'OFILE', required = True, default = argparse.SUPPRESS, help = 'name of the output file (required)')
+SnapCLParser.add_argument("-o", "--ofile", metavar = 'OFILE', dest = 'outputfile', required = True, default = argparse.SUPPRESS, help = 'name of the output file (required)')
 SnapCLParser.add_argument("--iformat", default = "bam", help = 'input file format; must be fastq, gz, sam or bam (default: bam)')
 SnapCLParser.add_argument("--oformat", default = "bam", help = 'output file format (sam or bam; default: bam)')
 SnapCLParser.add_argument('--header', default = argparse.SUPPRESS, help = 'a SAM file providing header information to be used for the output (required for input in fastq format and with unheadered SAM/BAM input, optional for headered SAM/BAM input; replaces header information found in the input file')
-SnapCLParser.add_argument("-d", "--maxdist", metavar = 'EDIT DISTANCE', type = int, default = argparse.SUPPRESS, help = 'maximum edit distance allowed per read or pair (default: 15)')
+SnapCLParser.add_argument("-d", "--maxdist", metavar = 'EDIT DISTANCE', type = int, default = 8, help = 'maximum edit distance allowed per read or pair (default: 8)')
 SnapCLParser.add_argument("-n", "--maxseeds", metavar = 'SEEDS', type = int, default = argparse.SUPPRESS, help = 'number of seeds to use per read (default: 25)')
 SnapCLParser.add_argument("-h", "--maxhits", metavar = 'HITS', type = int, default = argparse.SUPPRESS, help = 'maximum hits to consider per seed (default: 250)')
-SnapCLParser.add_argument("-c", "--confdiff", metavar = 'THRESHOLD', type = int, default = argparse.SUPPRESS, help = 'confidence threshold (default: 1)')
-SnapCLParser.add_argument("-a", "--confadapt", metavar = 'THRESHOLD', type = int, default = argparse.SUPPRESS, help = 'confidence adaptation threshold (default: 7)')
+SnapCLParser.add_argument("-c", "--confdiff", metavar = 'THRESHOLD', type = int, default = 2, help = 'confidence threshold (default: 2)')
+SnapCLParser.add_argument("-a", "--confadapt", metavar = 'THRESHOLD', type = int, default = 7, help = 'confidence adaptation threshold (default: 7)')
 SnapCLParser.add_argument("-t", "--threads", type = int, default = argparse.SUPPRESS, help = 'number of threads to use (overrides config setting)')
-SnapCLParser.add_argument("-b", "--bind_threads", action = 'store_true', default = argparse.SUPPRESS, help = 'bind each thread to its processor (off by default)')
-SnapCLParser.add_argument("-e", "--error_rep", action = 'store_true', default = argparse.SUPPRESS, help = 'compute error rate assuming wgsim-generated reads')
-SnapCLParser.add_argument('-P', '--no_prefetch', action = 'store_true', default = False, help = 'disables cache prefetching in the genome; may be helpful for machines with small caches or lots of cores/cache')
+SnapCLParser.add_argument("-b", "--bind-threads", dest = 'bind_threads', action = 'store_true', default = argparse.SUPPRESS, help = 'bind each thread to its processor (off by default)')
+SnapCLParser.add_argument("-e", "--error-rep", dest = 'error_rep', action = 'store_true', default = argparse.SUPPRESS, help = 'compute error rate assuming wgsim-generated reads')
+SnapCLParser.add_argument('-P', '--no-prefetch', dest = 'no_prefetch', action = 'store_true', default = False, help = 'disables cache prefetching in the genome; may be helpful for machines with small caches or lots of cores/cache')
 SnapCLParser.add_argument('--sort', '--so', metavar = 'MEMORY', nargs = '?', type = int, const=0, default = argparse.SUPPRESS, help = 'sort output file by alignment location; may be followed by the memory to use in Gb')
 SnapCLParser.add_argument("-x", "--explore", action = 'store_true', default = argparse.SUPPRESS, help = 'explore some hits of overly popular seeds (useful for filtering)')
-SnapCLParser.add_argument("-f", "--stop_on_first", action = 'store_true', default = argparse.SUPPRESS, help = 'stop on first match within edit distance limit (filtering mode)')
-SnapCLParser.add_argument("-F", "--filter_output", metavar = 'FILTER', default = argparse.SUPPRESS, help = 'retain only certain read classes in output (a=aligned only, s=single hit only, u=unaligned only)')
+SnapCLParser.add_argument("-f", '--stop-on-first', dest = 'stop_on_first', action = 'store_true', default = argparse.SUPPRESS, help = 'stop on first match within edit distance limit (filtering mode)')
+SnapCLParser.add_argument("-F", "--filter-output", dest = 'filter_output', metavar = 'FILTER', default = argparse.SUPPRESS, help = 'retain only certain read classes in output (a=aligned only, s=single hit only, u=unaligned only)')
 SnapCLParser.add_argument("-I", "--ignore", action = 'store_true', default = argparse.SUPPRESS, help = 'ignore non-matching IDs in the paired-end aligner')
 SnapCLParser.add_argument("-S", "--selectivity", type = int, default = argparse.SUPPRESS, help = 'selectivity; randomly choose 1/selectivity of the reads to score')
 SnapCLParser.add_argument("-C", "--clipping", metavar = '++|+-|-+|--', default = argparse.SUPPRESS, help = 'specify a combination of two + or - symbols to indicate whether to clip low-quality bases from the front and back of reads respectively; default: back only (-C-+)')
-SnapCLParser.add_argument("-M", "--mmatch_notation", action = 'store_true', default = argparse.SUPPRESS, help = 'indicates that CIGAR strings in the generated SAM file should use M (alignment match) rather than = and X (sequence (mis-)match)')
-SnapCLParser.add_argument("-G", "--gap_penalty", type = int, default = argparse.SUPPRESS, help = 'specify a gap penalty to use when generating CIGAR strings')
+SnapCLParser.add_argument("-M", "--mmatch-notation", dest = 'mmatch_notation', action = 'store_true', default = argparse.SUPPRESS, help = 'indicates that CIGAR strings in the generated SAM file should use M (alignment match) rather than = and X (sequence (mis-)match)')
+SnapCLParser.add_argument("-G", "--gap-penalty", dest = 'gap_penalty', metavar = 'PENALTY', type = int, default = argparse.SUPPRESS, help = 'specify a gap penalty to use when generating CIGAR strings')
 SnapCLParser.add_argument("-s", "--spacing", nargs = 2, metavar = ('MIN','MAX'), default = argparse.SUPPRESS, help = 'min and max spacing to allow between paired ends (default: 100 1000)')
-SnapCLParser.add_argument('--max_mate_overlap', '--mmo', metavar = 'FRACTION', type = float, default = argparse.SUPPRESS, help = 'for paired-end reads only retain reads with maximally this fraction of their length overlap with their mate; default: 0 (no overlap allowed)')
+SnapCLParser.add_argument('--max-mate-overlap', '--mmo', dest = 'max_mate_overlap', metavar = 'FRACTION', type = float, default = argparse.SUPPRESS, help = 'for paired-end reads only retain reads with maximally this fraction of their length overlap with their mate; default: 0 (no overlap allowed)')
 SnapCLParser.add_argument('-q', '--quiet', action = 'store_true', default = False, help = 'suppress original messages from SNAP')
 SnapCLParser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'verbose output (independent of SNAP)')
 idx_group = SnapCLParser.add_argument_group('optional arguments affecting indexing')
-idx_group.add_argument("--idx_seedsize", metavar = 'SEED SIZE', type = int, default = argparse.SUPPRESS, help = 'Seed size used in building the index (default: 20)')
-idx_group.add_argument("--idx_slack", metavar = 'SLACK', type = float, default = argparse.SUPPRESS, help = 'Hash table slack for indexing (default: 0.3)')
-idx_group.add_argument('--idx_out', default = argparse.SUPPRESS, help = 'name of the index directory to be created; if given, the index directory will be permanent, otherwise a temporary directory will be used')
-
+idx_group.add_argument("--idx-seedsize", dest = 'idx_seedsize', metavar = 'SEED SIZE', type = int, default = argparse.SUPPRESS, help = 'Seed size used in building the index (default: 20)')
+idx_group.add_argument("--idx-slack", dest = 'idx_slack', metavar = 'SLACK', type = float, default = argparse.SUPPRESS, help = 'Hash table slack for indexing (default: 0.3)')
+idx_group.add_argument('--idx-overflow', dest ='idx_ofactor', metavar = 'FACTOR', type = int, default = argparse.SUPPRESS, help = 'factor (between 1 and 1000) to set the size of the index build overflow space (default: 40)')
+idx_group.add_argument('--idx-out', metavar = 'INDEX DIR', dest = 'idx_out', default = argparse.SUPPRESS, help = 'name of the index directory to be created; if given, the index directory will be permanent, otherwise a temporary directory will be used')
 
 def clparse(args = None):
     """Command line parsing to make snap_call() usable from the terminal."""
@@ -571,11 +581,16 @@ def clparse(args = None):
     return vars(SnapCLParser.parse_args(args)).copy()
 
 
-def snap_index (ref_genome, index_out, seedsize = None, slack = None, threads = None, verbose = False, quiet = True):
+def snap_index (ref_genome, index_out, seedsize = None, slack = None, ofactor = None, threads = None, verbose = False, quiet = True):
     """A simple wrapper around SNAP index."""
 
     if not threads:
         threads = config.multithreading_level
+    if ofactor is not None:
+        if ofactor < 1:
+            ofactor = 1
+        elif ofactor > 1000:
+            ofactor = 1000
     with open(ref_genome, 'r') as ifo:
         md5sums = [md5 for md5 in FastaReader(ifo).md5sums()]
     call = '{0} index {1} {2} -t{3}'.format(snap_exe, ref_genome, index_out, threads)
@@ -583,7 +598,8 @@ def snap_index (ref_genome, index_out, seedsize = None, slack = None, threads =
         call = ' '.join((call, '-s', str(seedsize)))
     if slack:
         call = ' '.join((call, '-h', str(slack)))
-
+    if ofactor:
+        call = ' '.join((call, '-O{0}'.format(ofactor)))
     if verbose:
         print('Calling snap index with:')
         print(call)
